/**********************************************************************
Code associated to the paper:

Resolution of a large number of small random symmetric 
linear systems in single precision arithmetic on GPUs

by: 

Lokman A. Abbas-Turki and Stef Graillat

Those who re-use this code should mention in their code 
the name of the authors above.
**********************************************************************/

/**********************************************************************

Only factorization

**********************************************************************/

#define EPS (0.0000001f)

///////////////////////////////////////////////////////////////////////
// Tridiagonalization: one thread per matrix
// This is a straight modification of function tred2 in 
// Numerical Recipes in C
///////////////////////////////////////////////////////////////////////
__global__ void tred_fact_k(float *a, float *d, float *s, int n);


///////////////////////////////////////////////////////////////////////
// Tridiagonalization: The number of communicating threads per system 
// is equal to the size of the system. 
// This version performs much better than tred_k. 
///////////////////////////////////////////////////////////////////////
__global__ void tred_fact_max_k(float *a, float *d, float *s, int n);


///////////////////////////////////////////////////////////////////////
// Tridiagonalization: The number of communicating threads per system 
// is equal to ComT. This version performs better than tred_fact_k 
// but it is generally slower than tred_fact_max_k.
///////////////////////////////////////////////////////////////////////
__global__ void tred_fact_hyb_k(float *a, float *d, float *s, int n, 
								int ComT);


/**********************************************************************

System resolution tred_max_k

**********************************************************************/
///////////////////////////////////////////////////////////////////////
// Parallel cyclic reduction tridiagonal symmetric matrices
// + two matrix/vector products
///////////////////////////////////////////////////////////////////////
__global__ void pcr_sym_k(float *q, float *a, float *b, float *y, int n);