/**********************************************************************
Code associated to the paper:

Resolution of a large number of small random symmetric 
linear systems in single precision arithmetic on GPUs

by: 

Lokman A. Abbas-Turki and Stef Graillat

Those who re-use this code should mention in their code 
the name of the authors above.
**********************************************************************/

/**********************************************************************

Only factorization

**********************************************************************/
// Optimized for matrices smaller than 4x4: one thread per system
__global__ void LDLt_fact_k(float *a, int n);


// Optimized for matrices bigger than 64x64: x64 number of threads
__global__ void LDLt_fact_max_k(float *a, int n);


// Optimized for matrices bigger than 4x4 and smaller than 64x64
__global__ void LDLt_fact_hyb_k(float *a, float *y, int n, int ComT);


/**********************************************************************

System resolution

**********************************************************************/
// Optimized for matrices smaller than 4x4: one thread per system
__global__ void LDLt_k(float *a, float *y, int n);

// Optimized for matrices bigger than 64x64: x64 number of threads
__global__ void LDLt_max_k(float *a, float *y, int n);

// Optimized for matrices bigger than 4x4 and smaller than 64x64
__global__ void LDLt_hyb_k(float *a, float *y, int n, int ComT);