/**********************************************************************
Code associated to the paper:

Resolution of a large number of small random symmetric
linear systems in single precision arithmetic on GPUs

by:

Lokman A. Abbas-Turki and Stef Graillat

Those who re-use this code should mention in their code
the name of the authors above.
**********************************************************************/

// The minimum size of random numbers generated
// at the same time
#define M 1048576

// Details nedded when launching a kernel
#define BlockSize 1024
#define GridSize (M/BlockSize)

////////////////////////////////////////////////////////////////
// L'Eucuyer CMRG Matrix Values
////////////////////////////////////////////////////////////////
// First MRG 
#define a12 63308
#define a13 -183326
#define q12 33921
#define q13 11714
#define r12 12979
#define r13 2883

// Second MRG 
#define a21 86098
#define a23 -539608
#define q21 24919
#define q23 3976
#define r21 7417
#define r23 2071

// Normalization variables
#define Invmp   4.6566129e-10f                  
#define two17   131072.0
#define two53   9007199254740992.0

////////////////////////////////////////////////////////////////
// Datatype definitions
////////////////////////////////////////////////////////////////
// CMRG datatype
typedef int TabSeedCMRG_t[M][6];
typedef float Tab2RNG_t[M][2];

////////////////////////////////////////////////////////////////
// Memory for RNG use 
////////////////////////////////////////////////////////////////
// The state variables of CMRG on GPU 
extern TabSeedCMRG_t *pt_CMRG;
// The combination of CMRGs, the used RNGs
extern Tab2RNG_t *pt_2RNG;
// The state variables of CMRG on CPU
extern TabSeedCMRG_t *pt_CMRGCPU;
// Matrixes associated to the post treatment of the CMRG
// - First MRG
extern double A1[3][3];
// - Second MRG
extern double A2[3][3];

////////////////////////////////////////////////////////////////
// Generation of an array of two uniformly distributed variables
////////////////////////////////////////////////////////////////
void UniformRandom(void);

////////////////////////////////////////////////////////////////
// Post initialization of CMRG
////////////////////////////////////////////////////////////////
void PostInitDataCMRG(void);